"""

:platform: Unix, Windows 
:last changed: 2022-05-28

.. moduleauthor:: Tjark Leon Raphael Gröne <tgroene@physnet.uni-hamburg.de>


"""

import pathlib
rootdir = str(pathlib.Path(__file__).parent.resolve())

from scipy import io as sp_io
from time import time

try:
    from backbone.functions.plotting import Plottin_PDF
    from backbone.functions.export import Export_PDF_Plot
    from backbone.functions.import_data import Import_PDF_Plot
    from backbone.functions.utils import averageArray
except:
    from python_scripts.backbone.functions.plotting import Plottin_PDF
    from python_scripts.backbone.functions.export import Export_PDF_Plot
    from python_scripts.backbone.functions.import_data import Import_PDF_Plot
    from python_scripts.backbone.functions.utils import averageArray

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#
#S/////////////////////////////////////////////////////////////////////////////////////////////////////S#
#S//                                                                                                 //S#
#S//    |----------------------------------------------------------------------------------------|   //S#
#S//    |                                                                                        |   //S#
#S//    |                                 The Insitu PDF Class                                   |   //S#
#S//    |                                                                                        |   //S#
#S//    |----------------------------------------------------------------------------------------|   //S#
#S//                                                                                                 //S#
#S/////////////////////////////////////////////////////////////////////////////////////////////////////S#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~#


class InsituPDF(Plottin_PDF,Export_PDF_Plot,Import_PDF_Plot):
    #----------------------------------------------------------------------------------------#
    #                                           Init                                         #
    #----------------------------------------------------------------------------------------#
    def __init__(self, path: str,tempPath: str,sampleName: str,startScan: int,endScan: int, timeOfset=0, 
                figratio1D=[16,9], figratio2D=[7, 4], reload: str = None, average: int = 30, without_dark = True):
        """
        Arguments
        ----------
        path:           Path to file
        sampleName:     Name of your sample (for Ledgend)
        startScan:      First scan to include 
        endScan:        Last scan to include
        counterXAS:     counters for the Scan measurements    
        timeOfset:      Ofset of the timeaxes for insitu measurements in s (-10 time axis starts with -10 s)       
        figratio1D:     Ratio of the plot window for 1D plots
        figratio2D:     Ratio of the plot window for 2D plots
        linewidth:      Linewith for every 1D plot
        reload:         Path to a saved MAT file, that was saved with the saveReload() function before
        """
        self.t0 = time() #start time
        self.path = path
        self.sampleName = sampleName
        self.startScan = startScan
        self.endScan = endScan
        self.timeOfset= timeOfset
        self.without_dark = without_dark
        self.reload = reload
        self.average = average
        
        self.max_all_int = 0
        self.min_all_int = 0

        ratio1D = figratio1D[0]/figratio1D[1] 
        figratio1D = [ratio1D*9,9]
        self.figratio1D = figratio1D
        ratio2D = figratio2D[0]/figratio2D[1] 
        figratio2D = [ratio2D*4,4]
        self.figratio2D = figratio2D

        if reload != None:
            loadedmat = sp_io.loadmat(reload)
            self.G = loadedmat['G_of_r']
            self.r = loadedmat['r'][0]
            self.time_array = loadedmat['time'][0] + timeOfset
            self.time_averaged_array = loadedmat['time_averaged'][0][startScan:endScan] + timeOfset
            self.temp_array = loadedmat['temp'][0]
            self.temp_averaged_array = loadedmat['temp_averaged'][0][startScan:endScan]
        else:
            print("importing data")
            self.import_data()
            # self.getTempTime(tempPath)
            
            loadedmat = sp_io.loadmat(tempPath)
            self.time_array = loadedmat['time'][0] + timeOfset
            self.time_averaged_array = loadedmat['time_averaged'][0][startScan:endScan] + timeOfset
            self.temp_array = loadedmat['temp'][0]
            self.temp_averaged_array = loadedmat['temp_averaged'][0][startScan:endScan]
            self.time_averaged_array = averageArray(self.time_array,self.average)
            self.temp_averaged_array = averageArray(self.temp_array,self.average)
            # self.useAverage(average)
            self.temp_averaged_array = self.temp_averaged_array[:len(self.G)]
            self.time_averaged_array = self.time_averaged_array[:len(self.G)]

        self.G = self.G[startScan:endScan]
        
        print("\n Time for processing")
        print("------------------------------------------------------------------")
        print("Time: " + str((time() - self.t0)/60) + "\n")
        #self.logger['Time it took to load'] = {'Time in min': str((time() - self.t0)/60)}
            

